function [Z,Zt,t0,N,M]=paramSweepCodistDrange(mu,nu,tMax,theseAxes)
% [Z,Zt,t0,N,M]=paramSweepCodistDrange(mu,nu,tMax,theseAxes)
% spmd + codistributed + for-drange code for parameter sweep study 
% to investigate serial vs parallel
%
% This version provides a natural way to allocate space for the results and 
%  to partition the computation.
%
% There are two limitations for this problem, and there is a workaround for each:
%  1. Theres no way to build a codistributed NaT array directly, so we build from numeric NaN
%  2. The for-drange loop can only operate correctly if the distribution dimension 
%     is the last dimension, so we operate on vectors and then build Zt
%
% Based on:
%    https://www.mathworks.com/help/distcomp/examples/...
%    plot-progress-during-parallel-computations-using-parfor-and-dataqueue.html

% Copyright 2019 The MathWorks, Inc

if nargin<4
    theseAxes=[];
end

%% Set up problem to provide a single index to loop on
[M,N] = meshgrid(mu,nu);
Z  = NaN(size(N));

if ~isempty(theseAxes)
    visualizeParamSweep(M, N ,Z, theseAxes);
end

%% Parameter Sweep
t0 = getDatetime0;

D = parallel.pool.DataQueue;
D.afterEach(@(x) updateSurface(theseAxes,x));

% Number of elements
nn = numel(N);
spmd   
    ZtIn = datetime(codistributed.NaN(nn, 3))';
    ZtOut = datetime(codistributed.NaN(nn, 3))';
    Zw = zeros(1, numel(N), codistributor1d());
    for ii = drange(1:numel(N))
        ZtIn(ii) = datetime();
        Zw(ii)= getVdpPeriod(M(ii), N(ii), tMax);
        ZtOut(ii) = datetime();
        send(D, [ii Zw(ii)]);
    end
    Zt = cat(3, ZtIn', ZtOut');
end

Z = gather(Zw);
Zt = gather(Zt);

end