function [Z,Zt,t0,N,M]=paramSweepSPMDadvanced(mu,nu,tMax,theseAxes)
% [Z,Zt,t0,N,M]=paramSweepSPMDadvanced(mu,nu,tMax,theseAxes)
% alternate spmd code for parameter sweep study to investigate serial vs parallel
%
% Why use this advanced version:
%    All data is returned in DataQueue, which is much more memory efficient
%    See also notes in paramSweepSPMD
%
% Additionally...
% If M and N have same number of elements as 1:numel(N), which is the drange 
% argument, you could distribute them as Mw and Nw with the default
% distribution and save memory by using Mw(ii) and Nw(ii) in the spmd block.   
% The saved space would come with a cost of slower indexing, but this should 
% not be a problem if the computation time for getVdpPeriod is sufficiently large.
% For example Mw = distributed(M(:));
%
% Note: This code is dependent on using drange with variables that are the same size
% and use the same partitioning.
%
% Based on:
%    https://www.mathworks.com/help/distcomp/examples/...
%    plot-progress-during-parallel-computations-using-parfor-and-dataqueue.html

% Copyright 2019 The MathWorks, Inc

if nargin<4
    theseAxes=[];
end

%% Set up problem to provide a single index to loop on
[M,N] = meshgrid(mu,nu);

%% Data will all fit in memory of single worker, Workspace will be copied to each
Z  = NaN(size(N));
Zt = NaT(numel(N), 2);

if ~isempty(theseAxes)
    visualizeParamSweep(M, N , Z, theseAxes);
end

%% Parameter Sweep
t0 = getDatetime0;

D = parallel.pool.DataQueue;
D.afterEach(@nReceiveResult);

if isempty(gcp('nocreate'))
    error('paramSweepSPMD requires a pool of workers');
end

% In this version, the loop does not return anything
% All trnasfer is with DataQueue, which is much more memory efficient

spmd
    % Note: spmd only copies variables referenced inside the spmd block
        
    % drange uses the default distribution
    for ii = drange(1:numel(N))
        start = datetime;
        val = getVdpPeriod(M(ii), N(ii), tMax);
        finish = datetime;
        send(D, {ii val [start finish]});
    end
end

% Ensure that Zt is the same final shape as Z
Zt = reshape(Zt, size(N,1), size(N,2), 2);  % Note that the code analyzer makes Zt cyan - a hint to the nested function

    % Note: this is a nested function, which will have access to variables
    % in the workspace of teh function in which it's defined
    function nReceiveResult(data)
        [idx, zVal, times] = deal(data{:});
        updateSurface(theseAxes, [idx zVal]);
        Zt(idx, 1:2) = times;
        Z(idx) = zVal;
    end

end


